// on linux: g++ -O42 monotonic.cpp
// on windows: requires msvc 2015 or newer (accuracy is garbage on previous versions)

#include <cstdio>  // fprintf
#include <chrono>  // chrono::steady_clock

// create a reference point for all returned values;
// note that high_resolution_clock is monotonic on msvc2015 due to
// implementation details, not necessarily the case on linux,
// so let's use steady_clock which has slightly worse accuracy instead
static auto ref = std::chrono::steady_clock::now();

// this will not be the OS uptime like clock_gettime(monotonic),
// instead it counts from 0 when the program starts,
// guaranteed to be monotonic (always increasing)
unsigned long get_runtime() {
	auto now = std::chrono::steady_clock::now();
	return std::chrono::duration_cast<std::chrono::microseconds>(now-ref).count();
	// options: milliseconds microseconds nanoseconds
}

// this returns the time since "epoch",
// presumably boot time? depends, not specified in spec
unsigned long get_uptime() {
	auto now = std::chrono::steady_clock::now();
	return std::chrono::duration_cast<std::chrono::microseconds>(
		now.time_since_epoch()).count();
}

// test: dump the values as-is
void just_print() {
	while (true) {
		printf("%lu msec, %lu msec\n", get_runtime(), get_uptime());
	}
}

// if you need something more similar to the traditional timespec struct,
// split the value into seconds and nanoseconds, something like
void pretend_its_timespec() {
	while (true) {
		unsigned long measured = get_uptime();
		unsigned long tv_sec = measured / 1000000UL;
		unsigned long tv_nsec = (measured % 1000000UL) * 1000UL;
		printf("tv_sec %lu   tv_nsec %lu\n", tv_sec, tv_nsec);
	}
}

// test: check that it never decreases (OK on musl-gcc 8.3.0)
void check_monotonic() {
	unsigned long last_run, last_up;
	printf("now verifying that clock is monotonic; there should be no more prints\n");
	while (true) {
		auto t_run = get_runtime();
		auto t_up = get_uptime();
		if (t_run < last_run || t_up < last_up) {
			printf("%lu < %lu  OR  %lu < %lu\n", last_run, t_run, last_up, t_up);
		}
		last_run = t_run;
		last_up = t_up;
	}
}

int main(int argc, char** argv) {
	//check_monotonic();
	//pretend_its_timespec();
	just_print();
	return 0;
}

